<?php
/*
Plugin Name: Strip!
Plugin URI: http://rebelpixel.com/
Description: Removes hyperlink tags from a given comment.
Version: 0.2
Author: Markku Seguerra
Author URI: http://rebelpixel.com/projects/strip/
*/


/*  Copyright 2008  Markku Seguerra  (email : markku@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/* 
    This plugin was used as an example for my "Developing WordPress Plugins" talk
    at WordCamp Philippines 2008. I've tried documenting the functions and
    hooks so they may hopefully be useful to those who'd want to try
    developing their own plugins.

    If you have questions, feel free to email me at <markku@seguerra.com>
*/




/* 
    We call the add_filter hook 'comment_text' to add the button link we use
    to toggle a comment's status, allowing us to 'strip' or 'unstrip' links
    for that comment.
*/

add_filter('comment_text', 'strip_comments_add_admin');
function strip_comments_add_admin($text)
{
    global $parent_file, $comment;
    if ($parent_file == 'edit-comments.php') {
        return '<p class="rp-strip-p" id="p-'.$comment->comment_ID.'">'.strip_comments_link($comment->comment_ID).'</p>'.$text;
    } else {
        return $text;
    }
}


/* 
    We call the add_filter hook 'comment_text' to add the button link we use
    to toggle a comment's status, allowing us to 'strip' or 'unstrip' links
    for that comment.
*/

add_filter('comment_text', 'strip_comments_add_public', 999998);
function strip_comments_add_public($text)
{
    global $comment, $post;
   
    if ( current_user_can( 'moderate_comments', $post->ID ) && is_single() ) {
        return $text.'<p class="rp-strip-p" id="p-'.$comment->comment_ID.'">'.strip_comments_link($comment->comment_ID).'</p>';
    } else {
        return $text;
    }
}



/* 
    This function is responsible for showing the "strip" and "unstrip" button links
*/
function strip_comments_link($cid)
{
    $ban = get_option('strip-comments');
    if (!$ban || !in_array($cid, $ban)) {
        return '<em title="Remove links for this comment." href="javascript:;" onclick="strip_now('.$cid.');" class="rp-strip-do rp-strip-action">Strip!</em>';
    } else {
        return '<em title="Restore links for this comment." href="javascript:;" onclick="unstrip_now('.$cid.');" class="rp-strip-undo rp-strip-action">Unstrip!</em><br />(links not shown)';
    }
}








$strip_comments_list = array();
$strip_comments_list_loaded = 0;



/* 
    We need the 'comment_text' and 'get_comment_author_link' filters to apply
    our function to these items. Specifically, we remove the links from the
    author link and the comment itself.
*/
add_filter('comment_text', 'strip_comments_add',999999);
add_filter('get_comment_author_link', 'strip_comments_add');
function strip_comments_add($text)
{
    global $parent_file, $comment, $strip_comments_list, $strip_comments_list_loaded;
    if (!isset($parent_file)) {
        if (!$strip_comments_list_loaded) {
            $strip_comments_list = get_option('strip-comments');
            $strip_comments_list_loaded = 1;
        }
        if (is_array($strip_comments_list) && in_array($comment->comment_ID, $strip_comments_list)) {
            //return strip_tags($text);
            return strip_selected_tags($text, array('a'));
        } else {
            return $text;
        }
    } else {
        return $text;
    }
}








/* 
    the 'wp_ajax_' hooks are special actions to which you can attach
    functions but more importantly, you can prepend it with a variable
    name which you can through ajax as a GET parameter.
*/
add_action('wp_ajax_strip', 'do_strip');
add_action('wp_ajax_unstrip', 'do_unstrip');
function do_strip()
{
    $cid = strip_tags($_POST['c']);
    $ban = get_option('strip-comments');
    $ban[] = $cid;
    update_option('strip-comments', $ban);
    echo strip_comments_link($cid);
    exit;
}


function do_unstrip()
{
    $cid = strip_tags($_POST['c']);
    $ban = get_option('strip-comments');
    if (is_array($ban)) {
        foreach ($ban as $v) {
            if ($v != $cid) {
                $_new[] = $v;
            }
        }
    }
    update_option('strip-comments', $_new);
    echo strip_comments_link($cid);
    exit;
}



/* 
    This is our function that takes a string and removes html tags
    from it.
*/
function strip_selected_tags($text, $tags = array())
{
    $args = func_get_args();
    $text = array_shift($args);
    $tags = func_num_args() > 2 ? array_diff($args,array($text))  : (array)$tags;
    foreach ($tags as $tag){
        while(preg_match('/<'.$tag.'(|\W[^>]*)>(.*)<\/'. $tag .'>/iusU', $text, $found)){
            $text = str_replace($found[0],$found[2],$text);
        }
    }
    return preg_replace('/(<('.join('|',$tags).')(|\W.*)\/>)/iusU', '', $text);
}



/* 
    Calling the 'admin_head' action hook allows us to add certain blocks of code
    to the top of the admin section's markup. We use it to add our css rules
    as well as our javascript (ajax) functions that we call to toggle a
    comment's status.
*/
add_action('admin_head', 'strip_comments_head');
function strip_comments_head()
{
?>
<style type="text/css" media="screen">
<!--
.rp-strip-action {
    font-size:1.2em;
    letter-spacing:-1px;
}

.rp-strip-do,.rp-strip-undo {
    color:#fff !important;
    background:#900 !important;
    border:none !important;
    padding:2px 4px;
}
.rp-strip-undo {
    background:#090 !important;
}

.rp-strip-p {
    text-align:right;
    margin:-2em 0 0;
}

-->
</style>
<script type="text/javascript">
//<![CDATA[
function strip_now(cid) {
    jQuery.post("<?php echo get_option('siteurl'); ?>/wp-admin/admin-ajax.php",
        {action:"strip", "c":cid, "cookie": encodeURIComponent(document.cookie)},
        function(str)   {
            pn = '#p-' + cid;
            jQuery(pn).html(str);
        });
}

function unstrip_now(cid) {
    jQuery.post("<?php echo get_option('siteurl'); ?>/wp-admin/admin-ajax.php",
        {action:"unstrip", "c":cid, "cookie": encodeURIComponent(document.cookie)},
        function(str)   {
            pn = '#p-' + cid;
            jQuery(pn).html(str);
        });
}

//]]>
</script>



<?php
} // function




/* 
    Calling the 'admin_head' action hook allows us to add certain blocks of code
    to the top of the admin section's markup. We use it to add our css rules
    as well as our javascript (ajax) functions that we call to toggle a
    comment's status.
*/
add_action('wp_head', 'strip_comments_head_public');
function strip_comments_head_public()
{
    global $post;
    if (!current_user_can( 'moderate_comments', $post->ID ) || !is_single() )
        return;
    else {    
?>
<style type="text/css" media="screen">
<!--
.rp-strip-action {
    font-size:1.2em;
    letter-spacing:-1px;
}

.rp-strip-do,.rp-strip-undo {
    cursor:pointer;
    font-style:normal;
    color:#fff !important;
    background:#900 !important;
    border:none !important;
    padding:2px 4px;
}
.rp-strip-undo {
    background:#090 !important;
}

.rp-strip-p {
    text-align:right;
    /*margin:-2em 0 0;*/
}

-->
</style>
<script type="text/javascript">
//<![CDATA[
function strip_now(cid) {
    jQuery.post("<?php echo get_option('siteurl'); ?>/wp-admin/admin-ajax.php",
        {action:"strip", "c":cid, "cookie": encodeURIComponent(document.cookie)},
        function(str)   {
            pn = '#p-' + cid;
            jQuery(pn).html(str);
        });
}

function unstrip_now(cid) {
    jQuery.post("<?php echo get_option('siteurl'); ?>/wp-admin/admin-ajax.php",
        {action:"unstrip", "c":cid, "cookie": encodeURIComponent(document.cookie)},
        function(str)   {
            pn = '#p-' + cid;
            jQuery(pn).html(str);
        });
}

//]]>
</script>



<?php
    }
} // function

?>